@@ -17,6 +17,7 @@ class Device:
         'status':'',
         'neighbors':{},
         'os_version':'',
+        'lag_members':{},
         }
 
     def __init__(self, **kwargs):
@@ -130,20 +131,42 @@ class Rancid:
         self._remote = remote
         self._c = None
         if self._remote:
-            if priv_keyfile is not None:
-                self._c = Connection(
-                    rancid_host,
-                    user=remote_user,
-                    connect_kwargs={"key_filename":priv_keyfile},
-                    ).sftp()
 
-        elif not(remote_password) and remote_user == "rancid":
-            remote_password = get_env_vars(["RANCID_PASSWORD"])["RANCID_PASSWORD"]
-            self._c = Connection(
-                rancid_host,
-                user=remote_user,
-                connect_kwargs={"password":remote_password},
-                ).sftp()
+            success = False
+
+            # first try private keyfile
+            if priv_keyfile is not None:
+                try:
+                    self._c = Connection(
+                        rancid_host,
+                        user=remote_user,
+                        connect_kwargs={"key_filename":priv_keyfile},
+                        ).sftp()
+                    success = True
+                except Exception as e:
+                    logging.warning(f"keyfile {priv_keyfile} error: {e}")
+            
+            # next try remote user and password
+            if not(success) and remote_user:
+                try:
+                    # if user is rancid and no remote password provided, try pulling from env
+                    if remote_user == "rancid" and not(remote_password):
+                        remote_password = get_env_vars(["RANCID_PASSWORD"])["RANCID_PASSWORD"]
+
+                    self._c = Connection(
+                        rancid_host,
+                        user=remote_user,
+                        connect_kwargs={
+                            "password": remote_password,
+                            },
+                        ).sftp()
+                    success = True
+                except Exception as e:
+                    logging.warning(f"username/password error: {e}")
+            
+            # bail out if we could not connect
+            if not(success):
+                raise Exception(f"Could not connect to {rancid_host}")
 
         # first we're going to populate an internal device
         # list with Device objects from every router.db
@@ -183,7 +206,6 @@ class Rancid:
                     lines = fh.read().splitlines()  
             return lines
         except FileNotFoundError:
-            logging.error(f"Could not find file {file}")
             return []
 
     def _parse_rdb(self, db_file:str, active_only:bool=False) -> list:
@@ -211,7 +233,6 @@ class Rancid:
                 device.cfg_file = db_file.replace('router.db',f'configs/{fields[0]}')
                 devices.append(device)
                 
-
         return devices
 
     def _set_device_neighbors(self, device:Device):
@@ -223,9 +244,10 @@ class Rancid:
         Also shortens/cleans up port names to match their
         well-known abbreviations
         '''
-        file = f'{self.rancid_dir}/Topology/{device.ip}.ifTable'
 
+        file = f'{self.rancid_dir}/Topology/{device.rancid_name}.ifTable'
         lines = self.get_file_lines(file)
+
         ports = {}
         for l in lines:
             l = l.strip()
@@ -239,6 +261,24 @@ class Rancid:
 
         device.neighbors = ports
 
+    def _set_device_lag_members(self, device:Device):
+        '''
+        Looks for a topologywalker ".AGG" file for the device
+        and parses the data into a dict mapping 
+        '''
+        # first let's get LAG mappings
+        lag_file = f'{self.rancid_dir}/Topology/{device.rancid_name}.AGG'
+        pc_lines = self.get_file_lines(lag_file)
+        lag_members = {}
+        for line in pc_lines:
+            if len(re.split(":\s+",line)) == 2:
+                lag, members = re.split(":\s+",line)
+                if re.match(r'\S+', members):
+                    for member in members.split("\t"):
+                        lag_members[member] = lag
+
+        device.lag_members = lag_members
+
     def _parse_verstat(self) -> dict:
         '''
         Parses /home/rancid/VER/VERstat into a mapping of
@@ -274,6 +314,7 @@ class Rancid:
             device = devices[0]
             if self._get_neighbors:
                 self._set_device_neighbors(device)
+                self._set_device_lag_members(device)
 
             return device
 
@@ -296,6 +337,7 @@ class Rancid:
         if self._get_neighbors:
             for r in results:
                 self._set_device_neighbors(r)
+                self._set_device_neighbors(r)
 
         return results
  
@@ -320,6 +362,7 @@ class Rancid:
         to_do = devices.copy()
 
         while(len(to_do)):
+
             d = to_do.pop()
             if not(d.neighbors):
                 continue
